package com.sakurarealm.playerrisk;

import com.palmergames.bukkit.towny.event.PlayerEnterTownEvent;
import com.palmergames.bukkit.towny.event.PlayerLeaveTownEvent;
import com.sakurarealm.playerrisk.api.PlayerRiskManager;
import com.sakurarealm.playerrisk.api.PlayerRiskSettings;
import org.apache.commons.lang.NotImplementedException;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.plugin.java.JavaPlugin;

import java.util.*;

public final class PlayerRisk extends JavaPlugin implements Listener {

    PlayerRiskManager playerRiskManager = null;

    public PlayerRiskManager getPlayerRiskManager() {
        if (playerRiskManager == null) {
            throw new NullPointerException("PlayerRisk plugin is not initialized.");
        }
        return playerRiskManager;
    }

    public void load() {
        saveDefaultConfig();
        FileConfiguration config = getConfig();
        ConfigurationSection settingsSection = config.getConfigurationSection("settings");
        playerRiskManager = new PlayerRiskManager(this, new PlayerRiskSettings(settingsSection));
    }

    @Override
    public void onEnable() {
        // Plugin startup logic
        load();
        getCommand("playerrisk").setExecutor(new CommandHandler(this));
        //注册监听器
        Bukkit.getPluginManager().registerEvents(this,this);

        getServer().getScheduler().scheduleSyncRepeatingTask(this, this::addRiskToOutTownPlayers,
                0, playerRiskManager.settings.getIncreaseInterval()*20);
    }

    @Override
    public void onDisable() {
        // Plugin shutdown logic
    }


    /**
     * TODO: 当玩家进入城镇时, 危险度不再增加.
     *
     * @param event The PlayerEnterTownEvent.
     */
    @EventHandler
    public void onPlayerEnterTown(PlayerEnterTownEvent event) {
        //当玩家进入城镇后，移出map集合并且清零
        Player player = event.getPlayer();
        playerRiskManager.removePlayerLevel(player.getName());
        throw new NotImplementedException("Not implemented yet.");
    }

    /**
     * TODO: 当玩家离开城镇时, 危险度开始增加.
     *
     * @param event The PlayerLeaveTownEvent.
     */
    @EventHandler
    public void onPlayerLeaveTown(PlayerLeaveTownEvent event) {
        //当玩家离开城镇后,把玩家姓名放入mao中
        Player player = event.getPlayer();

        //此处应该有一个方法，用来获取生物群系的姓名或者id,在方法中默认只有一个群系,用字符a代替

        playerRiskManager.addPlayerToMap(player.getName(),"a");

        throw new NotImplementedException("Not implemented yet.");
    }

    /**
     * TODO: 当玩家死亡时, 危险度重置.
     *
     * @param event The PlayerDeathEvent.
     */
    @EventHandler
    public void onPlayerDeath(PlayerDeathEvent event) {
        Player player = event.getEntity();
        playerRiskManager.removePlayerLevel(player.getName());
        throw new NotImplementedException("Not implemented yet.");
    }
    // TODO: Add any other event handlers needed to manage player risk.

    /**
     * 此方法会每过60s执行一次,用于增加危险度
     */
    public void addRiskToOutTownPlayers(){
        //获取所有玩家的生物群系的危险度
        Map<String, Map<String, Integer>> playerBiomeRisk = playerRiskManager.getPlayerBiomeRisk();
        for (String name : playerBiomeRisk.keySet()){
            Map<String, Integer> biomeRisk = playerBiomeRisk.get(name);
            for (String biome : biomeRisk.keySet()){
                Integer level = biomeRisk.get(biome);
                if (level < playerRiskManager.settings.getLevelMax()/5){
                    //如果小于10则增加
                    playerRiskManager.setPlayerRiskLevel(name,biome,level+1);
                }
            }
        }
    }
}
